# coding=utf-8
from .utils import locale_to_str


__all__ = [
    'get_werkzeug_preferred_locales',
    'get_webob_preferred_locales',
    'get_django_preferred_locales',
]


def get_werkzeug_preferred_locales(request):
    """Return a list of preferred languages from a `werkzeug.wrappers.Request`
    instance.

    """
    languages = getattr(request, 'accept_languages', None)
    if languages:
        return [locale_to_str(l) for l in languages.values()]


def get_webob_preferred_locales(request):
    """Return a list of preferred languages from a `webob.Request` instance.

    """
    languages = getattr(request, 'accept_language', None)
    if languages:
        return [locale_to_str(l) for l in languages]


def get_django_preferred_locales(request):
    """Take a `django.HttpRequest` instance and return a list of preferred
    languages from the headers.

    """
    meta = getattr(request, 'META', None)
    if not meta:
        return None
    header = request.META.get('HTTP_ACCEPT_LANGUAGE')
    if header:
        languages = [l.strip().split(';')[::-1] for l in header.split(',')]
        languages = sorted(languages)[::-1]
        return [locale_to_str(l[1]) for l in languages]



# coding=utf-8
from babel import Locale
from babel.dates import UTC, get_timezone

from allspeak import RequestManager

from .conftest import make_get_request


def test_init():
    rm = RequestManager()
    assert rm.default_locale == Locale('en')
    assert rm.default_timezone == UTC


def test_set_defaults():
    rm = RequestManager(default_locale='es', default_timezone='America/Lima')
    assert rm.default_locale == Locale('es')
    assert rm.default_timezone == get_timezone('America/Lima')


def test_get_locale_default():
    locale = Locale('es')
    rm = RequestManager(default_locale=locale)
    assert rm.get_locale() == locale


def test_get_locale():
    locale = Locale('es', 'PE')
    default = Locale('en')
    available_locales = ['en', 'es_PE']
    rm = RequestManager(
        get_locale=lambda: locale, default_locale=default,
        available_locales=available_locales)
    assert rm.get_locale() == locale


def test_get_timezone_default():
    tzinfo = get_timezone('America/Lima')
    rm = RequestManager(default_timezone=tzinfo)
    assert rm.get_timezone() == tzinfo


def test_get_locale_from_request():
    default = Locale('ru')

    get_request = make_get_request(locale='es')
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['es']
    )
    assert rm.get_locale() == Locale('es')

    get_request = make_get_request(locale='en-US')
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['en-US']
    )
    assert rm.get_locale() == Locale('en', 'US')

    get_request = make_get_request(locale='en_US')
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['en_US']
    )
    assert rm.get_locale() == Locale('en', 'US')

    get_request = make_get_request(locale=Locale('en', 'US'))
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['en-US']
    )
    assert rm.get_locale() == Locale('en', 'US')

    get_request = make_get_request(locale=('es', ))
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['es']
    )
    assert rm.get_locale() == Locale('es')

    get_request = make_get_request(locale=('es', ))
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=[]
    )
    assert rm.get_locale() == default

    get_request = make_get_request(locale=('en', 'US'))
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['en-US']
    )
    assert rm.get_locale() == Locale('en', 'US')

    get_request = make_get_request(locale='En-Us')
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['En-Us']
    )
    assert rm.get_locale() == Locale('en', 'US')

    get_request = make_get_request(locale=None)
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['es']
    )
    assert rm.get_locale() == default

    get_request = make_get_request(locale='klingon')
    rm = RequestManager(
        get_request=get_request,
        default_locale=default,
        available_locales=['es']
    )
    assert rm.get_locale() == default


def test_get_locale_from_request_invalid():
    locale = None
    default = Locale('en')
    get_request = make_get_request(locale=locale)
    available_locales = ['en', 'es_PE']
    rm = RequestManager(
        get_request=get_request, default_locale=default,
        available_locales=available_locales)
    assert rm.get_locale() == default


def test_get_locale_from_request_unavailable():
    locale = None
    default = Locale('en')
    get_request = make_get_request(locale=locale)
    available_locales = []
    rm = RequestManager(
        get_request=get_request, default_locale=default,
        available_locales=available_locales)
    assert rm.get_locale() == default


def test_get_timezone_from_request():
    tzinfo = get_timezone('America/Lima')
    default = UTC
    get_request = make_get_request(tzinfo=tzinfo)
    rm = RequestManager(get_request=get_request, default_timezone=default)
    assert rm.get_timezone() == tzinfo


def test_get_timezone_from_request_invalid():
    tzinfo = None
    default = UTC
    get_request = make_get_request(tzinfo=tzinfo)
    rm = RequestManager(get_request=get_request, default_timezone=default)
    assert rm.get_timezone() == default


def test_get_timezone():
    tzinfo = get_timezone('America/Lima')
    rm = RequestManager(get_timezone=lambda: tzinfo, default_timezone=UTC)
    assert rm.get_timezone() == tzinfo



# coding=utf-8
from datetime import date, datetime, time, timedelta

from babel import Locale
from babel.dates import UTC, get_timezone

from allspeak import L10n

from .conftest import make_get_request


def test_init_l10n():
    l10n = L10n()
    assert l10n.default_locale == Locale('en')
    assert l10n.default_timezone == UTC
    assert repr(l10n) == 'L10n(default_locale=en, default_timezone=UTC)'


def test_init_l10n_defaults():
    l10n = L10n(default_locale='es', default_timezone='America/Lima')
    assert l10n.default_locale == Locale('es')
    assert l10n.default_timezone == get_timezone('America/Lima')


def test_to_user_timezone():
    l10n = L10n(default_timezone='America/Lima')
    now = datetime.utcnow().replace(hour=12)
    assert l10n.to_user_timezone(now).hour == now.hour - 5


def test_to_user_timezone_from_request():
    get_request = make_get_request(tzinfo='America/Lima')
    l10n = L10n(get_request=get_request)
    now = datetime.utcnow().replace(hour=12)
    result = l10n.to_user_timezone(now, )
    assert result.hour == now.hour - 5


def test_to_some_timezone():
    l10n = L10n()
    now = datetime.utcnow().replace(hour=12)
    result = l10n.to_user_timezone(now, tzinfo='America/Lima')
    assert result.hour == now.hour - 5


def test_to_some_timezone_obj():
    l10n = L10n()
    now = datetime.utcnow().replace(hour=12)
    tzinfo = get_timezone('America/Lima')
    result = l10n.to_user_timezone(now, tzinfo=tzinfo)
    assert result.hour == now.hour - 5


def test_to_utc():
    l10n = L10n()
    now = datetime.utcnow()
    # America/Lima = UTC-5
    dtz = l10n.to_user_timezone(now, tzinfo='America/Lima')
    assert l10n.to_utc(dtz) == now


def test_to_utc_use_default_tzinfo():
    # America/Lima = UTC-5
    l10n = L10n(default_timezone='America/Lima')
    now = datetime.utcnow()
    dt = now - timedelta(hours=5)
    assert dt.tzinfo is None
    assert l10n.to_utc(dt) == now


def test_format_date():
    l10n = L10n()
    d = date(2007, 4, 1)

    dformat = "EEE, MMM d, ''yy"
    expected = u"Sun, Apr 1, '07"
    assert l10n.format_date(d, dformat, locale='en') == expected

    # datetimes are converted to dates
    dt = datetime(2007, 4, 1, 15, 30)
    assert l10n.format_date(dt, dformat, locale='en') == expected


def test_format_date_by_name():
    l10n = L10n()
    d = date(2007, 4, 1)

    assert l10n.format_date(d, 'short', locale='en') == u'4/1/07'
    assert l10n.format_date(d, 'medium', locale='en') == u'Apr 1, 2007'
    assert l10n.format_date(d, 'long', locale='en') == u'April 1, 2007'
    assert l10n.format_date(d, 'full', locale='en') == u'Sunday, April 1, 2007'

    assert l10n.format_date(d, '...', locale='en') == u'...'
    assert l10n.format_date(d, locale='en') == u'Apr 1, 2007'


def test_set_date_formats():
    date_formats = {'date': 'short'}
    l10n = L10n(date_formats=date_formats)

    d = date(2007, 4, 1)
    assert l10n.format_date(d, locale='en') == u'4/1/07'

    l10n.set_date_formats({'date': 'full'})
    assert l10n.format_date(d, locale='en') == u'Sunday, April 1, 2007'


def test_format_datetime():
    l10n = L10n()
    dt = datetime(2007, 4, 1, 15, 30)

    dformat = "yyyyy.MMMM.dd GGG hh:mm a"
    expected = u'02007.April.01 AD 03:30 PM'
    assert l10n.format_datetime(dt, dformat, locale='en') == expected


def test_format_now():
    l10n = L10n()
    now = datetime.utcnow()
    dformat = "yyyy"
    expected = str(now.year)
    assert l10n.format_datetime('now', dformat) == expected


def test_format_datetime_tzinfo():
    l10n = L10n()
    dt = datetime(2007, 4, 1, 15, 30, tzinfo=UTC)
    eastern = get_timezone('US/Eastern')

    dformat = 'H:mm Z'
    expected = u'11:30 -0400'
    assert l10n.format_datetime(dt, dformat, tzinfo=eastern, locale='en_US') == expected


def test_format_time():
    l10n = L10n()
    t = time(15, 30)

    tformat = "hh 'o''clock' a"
    expected = u"03 o'clock PM"
    assert l10n.format_time(t, tformat, locale='en') == expected

    tformat = 'H:mm a'
    expected = u'15:30 nachm.'
    assert l10n.format_time(t, tformat, locale='de') == expected


def test_format_timedelta():
    l10n = L10n()
    delta = timedelta(days=6)

    expected = u'1 week'
    assert l10n.format_timedelta(delta, locale='en_US') == expected

    expected = u'1 month'
    assert l10n.format_timedelta(delta, granularity='month', locale='en_US') == expected

    # dates or datetimes are converted to timedelta from now
    dt = datetime.utcnow() + timedelta(days=7)
    expected = u'1 week'
    assert l10n.format_timedelta(dt, locale='en_US') == expected


def test_format_number():
    l10n = L10n()

    assert l10n.format_number(1099, locale='en_US') == u'1,099'
    assert l10n.format_number(1099, locale='de_DE') == u'1.099'


def test_format_decimal():
    l10n = L10n()

    assert l10n.format_decimal(1.2345, locale='en_US') == u'1.234'
    assert l10n.format_decimal(1.2345, locale='sv_SE') == u'1,234'
    assert l10n.format_decimal(12345, locale='de_DE') == u'12.345'
    assert l10n.format_decimal(-1.2345, format='#,##0.##;-#', locale='en') == u'-1.23'


def test_format_currency():
    l10n = L10n()

    assert (l10n.format_currency(1099.98, 'USD', locale='en_US') ==
            u'$1,099.98')
    assert (l10n.format_currency(1099.98, 'CAD', locale='en_US') ==
            u'CA$1,099.98')
    assert (l10n.format_currency(1099.98, 'EUR', locale='de_DE') ==
            u'1.099,98\xa0\u20ac')
    assert (l10n.format_currency(1099.98, 'EUR', locale='en_US') ==
            u'\u20ac1,099.98')
    assert (l10n.format_currency(1099.98, 'EUR', u'\xa4\xa4 #,##0.00', locale='en_US') ==
            u'EUR 1,099.98')


def test_format_percent():
    l10n = L10n()

    assert l10n.format_percent(0.34, locale='en_US') == u'34%'
    assert l10n.format_percent(25.1234, locale='en_US') == u'2,512%'
    assert l10n.format_percent(25.1234, locale='sv_SE') == u'2\xa0512\xa0%'
    assert l10n.format_percent(25.1234, u'#,##0\u2030', locale='en_US') == u'25,123\u2030'


def test_format_scientific():
    l10n = L10n()

    assert l10n.format_scientific(0.1, '#E0', locale='en_US') == '1E-1'
    assert l10n.format_scientific(-12345.6, '00.###E0', locale='en_US') == '-12.346E3'
    assert l10n.format_scientific(123.45, '#.##E0 m/s', locale='en_US') == '1.23E2 m/s'


def test_format():
    l10n = L10n()

    d = date(2007, 4, 1)
    dformat = "EEE, MMM d, ''yy"
    expected = u"Sun, Apr 1, '07"
    assert l10n.format(d, dformat, locale='en') == expected

    dt = datetime(2007, 4, 1, 15, 30)
    dformat = "yyyyy.MMMM.dd GGG hh:mm a"
    expected = u'02007.April.01 AD 03:30 PM'
    assert l10n.format(dt, dformat, locale='en') == expected

    dt = datetime(2007, 4, 1, 15, 30, tzinfo=UTC)
    eastern = get_timezone('US/Eastern')
    dformat = 'H:mm Z'
    expected = u'11:30 -0400'
    assert l10n.format(dt, dformat, tzinfo=eastern, locale='en_US') == expected

    t = time(15, 30)
    tformat = "hh 'o''clock' a"
    expected = u"03 o'clock PM"
    assert l10n.format(t, tformat, locale='en') == expected

    delta = timedelta(days=6)
    assert l10n.format(delta, locale='en_US') == u'1 week'

    assert l10n.format(delta, granularity='month', locale='en_US') == u'1 month'

    assert l10n.format(1099, locale='en_US') == u'1,099'

    assert l10n.format(1.2345, locale='en_US') == u'1.234'

    assert l10n.format(None, locale='en_US') == ''



# coding=utf-8
from babel import Locale
from markupsafe import Markup

from . import utils
from ._compat import string_types
from .reader import Reader
from .request_manager import RequestManager


class I18n(RequestManager):
    """Internationalization functions.

    Uses the :class:`Reader` class to load and parse the translations files.

    :param folderpath: path that will be searched for the translations.

    :param get_locale: a callable that returns the current locale

    :param default_locale: default locale (as a string or as a
        Babel.Locale instance). This value will be accepted
        without checking if it's available.

    :param markup: overwrite the function used by `translate` to flags HTML
        code as 'safe'. `markupsafe.Markup` is used by default.

    """

    def __init__(self, folderpath=utils.LOCALES_FOLDER, markup=Markup, **kwargs):
        self.reader = Reader(folderpath)
        self.markup = markup
        self.load_translations()
        super(I18n, self).__init__(**kwargs)
        self._set_available_locales(self.translations.keys())

    def __repr__(self):
        return '{cname}()'.format(
            cname=self.__class__.__name__,
        )

    def __call__(self, *args, **kwargs):
        """Calling this instance is a shortcut to calling ``self.translate``.
        Useful when translating Sphinx documentation, that pickle the environment
        (a method of an instance isn't pickable, but an instance of a class is).
        """
        return self.translate(*args, **kwargs)

    def load_translations(self, *locales):
        self.translations = self.reader.load_translations(locales=locales)

    def get_translations_from_locale(self, locale):
        """Return the available translations for a locale: the
        country-specific (is defined) and the one for the language in general.

        :param locale: must be a :class:`babel.core.Locale` instance or a
            string.
        """
        strlocale = utils.locale_to_str(locale)
        if not self.translations or strlocale not in self.translations:
            self.load_translations(locale)

        objs = []
        trans = self.translations.get(strlocale)
        if trans:
            objs.append(trans)
        if '_' not in strlocale:
            return objs

        strlocale = strlocale.split('_')[0]
        trans = self.translations.get(strlocale)
        if trans:
            objs.append(trans)
        return objs

    def key_lookup(self, locale, key):
        """Return the value of the translation for the given key using the
        current locale. It tries first with the country-specific (eg `en-US`)
        translations (if the locale it's that specific) and then with those
        of the general language (eg. `en`).

        :param locale: must be a :class:`babel.core.Locale` instance or a
            string.
        :param key: a string, the ID of the looked up translation
        """
        translations = self.get_translations_from_locale(locale)
        if not translations:
            # Language not found!
            return None

        for trans in translations:
            for subkey in key.split('.'):
                value = trans.get(subkey)
                if value is None:
                    break
                trans = value
            if value is not None:
                return value
        return None

    def translate(self, key, count=None, locale=None, **kwargs):
        """Get the translation for the given key using the current locale.

        If the value is a dictionary, and `count` is defined, uses the value
        whose key is that number. If that key doesn't exist, a `'n'` key
        is tried instead. If that doesn't exits either, an empty string is
        returned.

        The final value is formatted using `kwargs` (and also `count` if
        available) so the format placeholders must be named instead of
        positional.

        If the value isn't a dictionary or a string, is returned as is.

        Examples:

            >>> translate('hello_world')
            'hello {what}'
            >>> translate('hello_world', what='world')
            'hello world'
            >>> translate('a_list', what='world')
            ['a', 'b', 'c']

        :param key: a string, the ID of the looked up translation
        :param count: If the value is a dictionary, and `count` is defined,
            uses the value whose key is that number. If that key doesn't exist,
            a `'n'` key is tried instead. If that doesn't exits either, an
            empty string is returned.
        :param locale: must be a :class:`babel.core.Locale` instance or a
            string.
        :param **kwargs: for string interpolation of the value.

        """
        key = str(key)
        locale = utils.normalize_locale(locale) or self.get_locale()
        value = self.key_lookup(locale, key)
        if value is None:
            return self.markup('<missing:{0}/>'.format(key))

        if isinstance(value, dict):
            value = pluralize(value, count, locale)

        if isinstance(value, string_types):
            kwargs.setdefault('count', count)
            value = value.format(**kwargs)
            if key.endswith('_html'):
                return self.markup(value)

        return value

    @property
    def lazy_translate(self):
        class LazyWrapper(object):
            def __init__(self_, *args, **kwargs):
                self_.args = args
                self_.kwargs = kwargs

            def __repr__(self_):
                return self.translate(*self_.args, **self_.kwargs)

        return LazyWrapper

    def _set_available_locales(self, available_locales):
        _available = []
        for locale in available_locales or [self.default_locale]:
            lparts = utils.split_locale(locale)

            lp = '_'.join(lparts)
            if lp not in _available:
                _available.append(lp)

            if len(lparts) > 1:
                if lparts[0] not in _available:
                    _available.append(lparts[0])
        self.available_locales = _available

    def negotiate_locale(self, preferred, available=None):
        """From the available locales, negotiate the most adequate for the
        client, based on the "accept language" header.

        :param preferred: list of the user's preferred locales
            (as ISO 639-1 language codes or as Babel.Locale instances).

        :param available: optional list of available locales
            (as ISO 639-1 language codes or as Babel.Locale instances).
            If provided, overwrites the list of detected locales from
            the files in ``folderpath``.

        """
        available = available or self.available_locales or []
        preferred = map(utils.locale_to_str, preferred)
        available = map(utils.locale_to_str, available)
        # To ensure a consistent matching, Babel algorithm is used.
        return Locale.negotiate(preferred, available, sep='_')

    def test_for_incomplete_locales(self, *locales):
        """Check a list of locales for keys that are defined in one but not in
        the other.

        :param locales: two or more locales as strings. If not provided, all
            of the available locales are tested.

        :return: a dictionary with strlocales as keys and sets of missing
            keys for those locales as values.

        """
        if not self.translations:
            self.load_translations(*locales)
        if not locales:
            locales = self.translations.keys()
        locales = [utils.locale_to_str(locale) for locale in locales]

        all_keys = []
        keys = {}
        for strlocale in locales:
            trans_keys = utils._flatten(self.translations.get(strlocale)).keys()
            keys[strlocale] = set(trans_keys)
            all_keys.extend(trans_keys)

        all_keys = set(all_keys)
        missing_keys = {}
        for key, value in keys.items():
            missing = all_keys - value
            if missing:
                missing_keys[key] = missing

        return missing_keys


def pluralize(dic, count, locale=utils.DEFAULT_LOCALE):
    """Takes a dictionary and a number and return the value whose key in
    the dictionary is either

        a. that number, or
        b. the textual representation of that number according to the `CLDR
           rules <cldr_rules>`_ for that locale, Depending of the language, this can be:
           "zero", "one", "two", "few", "many" or "other".

    ..  rules: http://www.unicode.org/cldr/charts/latest/supplemental/language_plural_rules.html

    As a deviation of the standard:

    - If ``count`` is 0, a `'zero'` is tried
    - If the textual representation is `'other'` but that key doesn't exists, a
      `'many'` key is tried instead.

    Finally, if none of these exits, an empty string is returned.

    Examples:

    >>> dic = {
            0: u'No apples',
            1: u'One apple',
            3: u'Few apples',
            'many': u'{count} apples',
        }
    >>> pluralize(dic, 0)
    'No apples'
    >>> pluralize(dic, 1)
    'One apple'
    >>> pluralize(dic, 3)
    'Few apples'
    >>> pluralize(dic, 10)
    '{count} apples'

    >>> dic = {
            'zero': u'No apples whatsoever',
            'one': u'One apple',
            'other': u'{count} apples',
        }
    >>> pluralize(dic, 0)
    u'No apples whatsoever'
    >>> pluralize(dic, 1)
    'One apple'
    >>> pluralize(dic, 2)
    '{count} apples'
    >>> pluralize(dic, 10)
    '{count} apples'

    >>> pluralize({0: 'off', 'many': 'on'}, 3)
    'on'
    >>> pluralize({0: 'off', 'other': 'on'}, 0)
    'off'
    >>> pluralize({0: 'off', 'other': 'on'}, 456)
    'on'
    >>> pluralize({}, 3)


    Note that this function **does not** interpolate the string, just returns
    the right one for the value of ``count``.
    """
    count = int(count or 0)
    scount = str(count).strip()
    plural = dic.get(count, dic.get(scount))
    if plural is not None:
        return plural

    if count == 0:
        plural = dic.get('zero')
        if plural is not None:
            return plural

    if isinstance(locale, string_types):
        locale = Locale(locale)
    literal = locale.plural_form(count)
    return dic.get(literal, dic.get('many', u''))



# coding=utf-8
from __future__ import print_function

from allspeak import utils
from babel import Locale
from pytz import timezone

from .conftest import get_test_request


def test_split_locale():
    assert utils.split_locale('en-US') == ('en', 'US')
    assert utils.split_locale('En_us') == ('en', 'US')
    assert utils.split_locale(Locale('en', 'US')) == ('en', 'US')


def test_normalize_locale():
    assert utils.normalize_locale('es') == Locale('es')
    assert utils.normalize_locale('en-US') == Locale('en', 'US')
    assert utils.normalize_locale('en_us') == Locale('en', 'US')
    assert utils.normalize_locale('en-us') == Locale('en', 'US')
    assert utils.normalize_locale('en_US') == Locale('en', 'US')
    assert utils.normalize_locale('En-Us') == Locale('en', 'US')

    assert utils.normalize_locale(Locale('en', 'US')) == Locale('en', 'US')

    assert utils.normalize_locale(('es', )) == Locale('es')
    assert utils.normalize_locale(('en', 'US')) == Locale('en', 'US')
    assert utils.normalize_locale(['en', 'US']) == Locale('en', 'US')
    assert utils.normalize_locale(['EN', 'us']) == Locale('en', 'US')
    assert utils.normalize_locale(['en', 'US', 'Texas']) == Locale('en', 'US')

    assert utils.normalize_locale('klingon') is None
    assert utils.normalize_locale(None) is None
    assert utils.normalize_locale(1) is None


def test_normalize_timezone():
    assert utils.normalize_timezone(timezone('America/Lima')) == timezone('America/Lima')
    assert utils.normalize_timezone('America/Lima') == timezone('America/Lima')
    assert utils.normalize_timezone('Mars') is None


def test_locale_to_str():
    assert utils.locale_to_str(Locale('en', 'US')) == 'en_US'
    assert utils.locale_to_str(Locale('es')) == 'es'
    assert utils.locale_to_str('en_US') == 'en_US'


def test_flatten():
    dic = {
        'a': 1,
        'c': {
            'a': 2,
            'b': {
                'x': 5,
                'y': 10,
            }
        },
        'd': [1, 2, 3],
    }
    expected = {'a': 1, 'c.a': 2, 'c.b.x': 5, 'c.b.y': 10, 'd': [1, 2, 3]}
    assert utils._flatten(dic) == expected



# coding=utf-8
import fnmatch
import io
import os
from os.path import join, dirname, realpath, abspath, normpath, isdir, splitext

import yaml

from .utils import LOCALES_FOLDER, split_locale, _is_sequence


def get_yaml_data(filepath):
    """Parse a yaml locale file.
    """
    with io.open(filepath, mode='r', encoding='utf8') as f:
        data = yaml.safe_load(f)
    return data


class Reader(object):
    """Functions related to loading and parsing translation files.

    :param folderpath: path or a list of paths (relative or absolute) that will
        be searched for the translations.

    """
    def __init__(self, folderpath=LOCALES_FOLDER):
        self.folderpath = self._process_folderpath(folderpath)
        self._set_loaders()

    def __repr__(self):
        return '{cname}()'.format(
            cname=self.__class__.__name__,
        )

    def _set_loaders(self):
        self.loaders = {}
        self.loaders_ext = []
        self.register_loader('yml', get_yaml_data)

    def _process_folderpath(self, folderpath):
        if not _is_sequence(folderpath):
            folderpath = [folderpath]
        paths = []
        for path in folderpath:
            path = normpath(abspath(realpath(path)))
            if not isdir(path):
                path = dirname(path)
            paths.append(path)
        return paths

    def register_loader(self, ext, func):
        """Register a loader for a file extension.
        `func` must take a single argument with the full path of a
        locale file and return a dictionary with the data.
        """
        if ext not in self.loaders_ext:
            self.loaders_ext.append(ext)
        self.loaders[ext] = func

    def get_loader(self, filepath):
        """Get the file loader suitable for a specific file.
        """
        _, ext = splitext(filepath)
        ext = ext.strip('.')
        loader = self.loaders.get(ext)
        assert loader, "Don't known how to read `*.{ext}` files".format(ext=ext)
        return loader

    def _extract_locales(self, data):
        """Parse the translation data and return tuples with the first-level
        keys with the locale and it's children (the translations).
        """
        return [
            (
                '_'.join(split_locale(locale)),
                trans
            )
            for locale, trans in data.items()
        ]

    def _load_file(self, filepath):
        """Load and parse the locale file from filepath.
        `filepath` should be an absolute path.
        """
        loader = self.get_loader(filepath)
        data = loader(filepath)
        return self._extract_locales(data)

    def _update_translations(self, translations, filepath):
        """Update the `translations` dictionary with the translation data
        extracted from the file in `filepath`.
        """
        data = self._load_file(filepath)
        for locale, trans in data:
            translations.setdefault(locale, {})
            translations[locale].update(trans)

    def load_translations(self, folderpath=None, locales=None):
        """Search for locale files on `folderpath`,
        load and parse them to build a big dictionary with all the
        translations data.

        Only the files with an extension listed in ``loaders_ext`` are loaded
        because only them have a registered loader.

        :param folderpath: overwrite the stored locales folder or list of folders.

        :param locales: does nothing, but might be useful to implement
            load-on-demand in your own subclass.

        """
        if folderpath:
            folderpath = self._process_folderpath(folderpath)
        else:
            folderpath = self.folderpath

        translations = {}
        for path in folderpath:
            for root, dirnames, filenames in os.walk(path):
                for ext in self.loaders_ext:
                    pattern = u'*.{}'.format(ext)
                    for filename in fnmatch.filter(filenames, pattern):
                        if filename.startswith('.'):
                            continue
                        filepath = join(root, filename)
                        self._update_translations(translations, filepath)

        return translations



# coding=utf-8
from __future__ import print_function

from allspeak import pluralize
from babel import Locale


def test_pluralize_numbers():
    d = {
        0: u'No apples',
        1: u'One apple',
        3: u'Few apples',
        'other': u'{count} apples',
    }
    assert pluralize(d, 0) == u'No apples'
    assert pluralize(d, 1) == u'One apple'
    assert pluralize(d, 3) == u'Few apples'
    assert pluralize(d, 10) == u'{count} apples'


def test_pluralize_literal():
    d = {
        'zero': u'No apples',
        'one': u'One apple',
        'few': u'Few apples',
        'many': u'{count} apples',
    }
    assert pluralize(d, 0) == u'No apples'
    assert pluralize(d, 1) == u'One apple'
    assert pluralize(d, 3) == u'{count} apples'
    assert pluralize(d, 10) == u'{count} apples'


def test_pluralize_mixed():
    d = {
        'one': u'One apple',
        2: u'Two apples',
        'other': u'{count} apples',
    }
    assert pluralize(d, 1) == u'One apple'
    assert pluralize(d, 2) == u'Two apples'
    assert pluralize(d, 10) == u'{count} apples'


def test_pluralize_zero_or_many():
    d = {
        'zero': u'off',
        'many': u'on'
    }
    assert pluralize(d, 3) == u'on'

    d = {
        'zero': u'off',
        'many': u'on'
    }
    assert pluralize(d, 0) == u'off'
    assert pluralize(d, None) == u'off'
    assert pluralize({}, 3) == u''


def test_pluralize_other():
    d = {
        'one': u'One apple',
        'other': u'meh',
    }
    assert pluralize(d, 0) == u'meh'
    assert pluralize(d, 1) == u'One apple'
    assert pluralize(d, 2) == u'meh'
    assert pluralize(d, 3) == u'meh'
    assert pluralize(d, 10) == u'meh'


def test_two_plural_mode():
    d = {
        'zero': u'zero',
        'one': u'one',
        'two': u'two',
        'few': u'few',
        'other': u'other',
    }
    locale = Locale('en')

    assert pluralize(d, 0, locale) == u'zero'
    assert pluralize(d, 1, locale) == u'one'

    assert pluralize(d, 2, locale) == u'other'
    assert pluralize(d, 3, locale) == u'other'
    assert pluralize(d, 4, locale) == u'other'
    assert pluralize(d, 5, locale) == u'other'
    assert pluralize(d, 6, locale) == u'other'
    assert pluralize(d, 7, locale) == u'other'
    assert pluralize(d, 10, locale) == u'other'
    assert pluralize(d, 11, locale) == u'other'
    assert pluralize(d, 50, locale) == u'other'
    assert pluralize(d, 99, locale) == u'other'
    assert pluralize(d, 101, locale) == u'other'
    assert pluralize(d, 102, locale) == u'other'
    assert pluralize(d, 105, locale) == u'other'


def test_one_plural_mode():
    d = {
        'one': u'one',
        'two': u'two',
        'few': u'few',
        'many': u'many',
        'other': u'other',
    }
    locale = Locale('zh')

    assert pluralize(d, 0, locale) == u'other'
    assert pluralize(d, 1, locale) == u'other'
    assert pluralize(d, 2, locale) == u'other'
    assert pluralize(d, 3, locale) == u'other'
    assert pluralize(d, 4, locale) == u'other'
    assert pluralize(d, 5, locale) == u'other'
    assert pluralize(d, 6, locale) == u'other'
    assert pluralize(d, 7, locale) == u'other'
    assert pluralize(d, 10, locale) == u'other'
    assert pluralize(d, 11, locale) == u'other'
    assert pluralize(d, 50, locale) == u'other'
    assert pluralize(d, 99, locale) == u'other'
    assert pluralize(d, 101, locale) == u'other'
    assert pluralize(d, 102, locale) == u'other'
    assert pluralize(d, 105, locale) == u'other'

    d = {
        'zero': u'zero',
        'one': u'one',
        'two': u'two',
        'few': u'few',
        'many': u'many',
        'other': u'other',
    }
    locale = Locale('zh')

    assert pluralize(d, 0, locale) == u'zero'


def test_pluralize_arabic():
    d = {
        'zero': u'zero',
        'one': u'one',
        'two': u'two',
        'few': u'few',
        'many': u'many',
        'other': u'other',
    }
    locale = Locale('ar')

    assert pluralize(d, 0, locale) == u'zero'
    assert pluralize(d, 1, locale) == u'one'
    assert pluralize(d, 2, locale) == u'two'

    assert pluralize(d, 3, locale) == u'few'
    assert pluralize(d, 4, locale) == u'few'
    assert pluralize(d, 5, locale) == u'few'
    assert pluralize(d, 6, locale) == u'few'
    assert pluralize(d, 7, locale) == u'few'
    assert pluralize(d, 10, locale) == u'few'

    assert pluralize(d, 11, locale) == u'many'
    assert pluralize(d, 50, locale) == u'many'
    assert pluralize(d, 99, locale) == u'many'

    assert pluralize(d, 101, locale) == u'other'
    assert pluralize(d, 102, locale) == u'other'
    assert pluralize(d, 105, locale) == u'few'


def test_pluralize_russian():
    d = {
        'zero': u'zero',
        'one': u'one',
        'two': u'two',
        'few': u'few',
        'many': u'many',
        'other': u'other',
    }
    locale = Locale('ru')

    assert pluralize(d, 0, locale) == u'zero'
    assert pluralize(d, 1, locale) == u'one'

    assert pluralize(d, 2, locale) == u'few'
    assert pluralize(d, 3, locale) == u'few'
    assert pluralize(d, 4, locale) == u'few'

    assert pluralize(d, 5, locale) == u'many'
    assert pluralize(d, 6, locale) == u'many'
    assert pluralize(d, 7, locale) == u'many'
    assert pluralize(d, 10, locale) == u'many'
    assert pluralize(d, 11, locale) == u'many'

    assert pluralize(d, 21, locale) == u'one'
    assert pluralize(d, 22, locale) == u'few'
    assert pluralize(d, 23, locale) == u'few'
    assert pluralize(d, 24, locale) == u'few'
    assert pluralize(d, 25, locale) == u'many'

    assert pluralize(d, 50, locale) == u'many'
    assert pluralize(d, 99, locale) == u'many'

    assert pluralize(d, 101, locale) == u'one'
    assert pluralize(d, 102, locale) == u'few'
    assert pluralize(d, 105, locale) == u'many'

    assert pluralize(d, 111, locale) == u'many'
    assert pluralize(d, 112, locale) == u'many'
    assert pluralize(d, 113, locale) == u'many'
    assert pluralize(d, 114, locale) == u'many'
    assert pluralize(d, 119, locale) == u'many'

    assert pluralize(d, 121, locale) == u'one'
    assert pluralize(d, 122, locale) == u'few'
    assert pluralize(d, 125, locale) == u'many'



# coding=utf-8
from babel import Locale
from babel.dates import get_timezone

from . import utils
from .utils import DEFAULT_LOCALE, DEFAULT_TIMEZONE
from .integrations import (
    get_werkzeug_preferred_locales,
    get_webob_preferred_locales,
    get_django_preferred_locales,
)


class RequestManager(object):
    """
    A base class with methods for getting the locale and timezone.

    :param get_locale: a callable that returns the current locale

    :param get_timezone: a callable that returns the current timezone

    :param default_locale: default locale (as a string or as a
        Babel.Locale instance). This value will be accepted
        without checking if it's available.

    :param default_timezone: default timezone (as a string or as a
        `datetime.tzinfo` instance).

    :param get_request: a callable that returns the current request.
        Do not use this, it exist only for backwards compatibility.

    """
    DEFAULT_DATE_FORMATS = {
        'time': 'medium',
        'date': 'medium',
        'datetime': 'medium',
    }

    def __init__(self, get_locale=None, get_timezone=None,
                 default_locale=DEFAULT_LOCALE,
                 default_timezone=DEFAULT_TIMEZONE,
                 get_request=None, available_locales=None):
        self._get_locale = get_locale
        self._get_timezone = get_timezone

        self.set_defaults(default_locale, default_timezone)
        self.translations = {}

        # Deprecated
        self._get_request = get_request
        self.available_locales = available_locales

    def __repr__(self):
        return '{cname}(default_locale={default_locale}, default_timezone={default_timezone})'.format(
            cname=self.__class__.__name__,
            default_locale=self.default_locale,
            default_timezone=self.default_timezone,
        )

    def set_defaults(self, default_locale, default_timezone):
        """Set the default locale from the configuration as an instance of
        :class:`babel.core.Locale` and the default timezone as a
        `datetime.tzinfo`.

        """
        self.default_locale = (
            utils.normalize_locale(default_locale) or
            Locale(DEFAULT_LOCALE)
        )
        self.default_timezone = (
            utils.normalize_timezone(default_timezone) or
            get_timezone(DEFAULT_TIMEZONE)
        )

    def get_locale(self):
        if self._get_locale:
            return self._get_locale()
        # deprecated
        if self._get_request:
            return self._deprecated_get_locale_from_request()
        #
        return self.default_locale

    def get_timezone(self):
        if self._get_timezone:
            return self._get_timezone()
        # deprecated
        if self._get_request:
            return self._deprecated_get_timezone_from_request()
        #
        return self.default_timezone

    def _deprecated_get_locale_from_request(self):
        """Set the locale that should be used for this request as a
        `babel.Locale` instance.

        Tries the following in order:

        - an request attribute called `'locale'`
        - a GET argument called `'locale'`
        - the prefered
        - the default locale

        """
        request = self._get_request()
        if not request:
            return self.default_locale
        if not self.available_locales:
            return self.default_locale
        locale = (
            getattr(request, 'locale', None) or
            getattr(request, 'args', getattr(request, 'GET', {})).get('locale')
        )
        if locale:
            preferred = [locale]
        else:
            preferred = (
                get_werkzeug_preferred_locales(request) or
                get_webob_preferred_locales(request) or
                get_django_preferred_locales(request) or
                []
            )

        preferred = map(utils.locale_to_str, preferred)
        available = map(utils.locale_to_str, self.available_locales)
        # To ensure a consistent matching, Babel algorithm is used.
        locale = Locale.negotiate(preferred, available, sep='_')
        request.locale = locale or self.default_locale
        return request.locale

    def _deprecated_get_timezone_from_request(self):
        """Set the timezone that should be used for this request as a
        `datetime.tzinfo` instance.

        Tries the following in order:

        - an attribute called `'tzinfo'`
        - a GET argument called `'tzinfo'`
        - the provided default timezone
        """
        request = self._get_request()
        if not request:
            return self.default_timezone
        tzinfo = (
            getattr(request, 'tzinfo', None) or
            getattr(request, 'args', getattr(
                    request, 'GET', {})).get('tzinfo')
        )
        request.tzinfo = utils.normalize_timezone(tzinfo) or self.default_timezone
        return request.tzinfo



# coding=utf-8
from .i18n import I18n
from .l10n import L10n


class Allspeak(I18n, L10n):
    """
    :param folderpath: path that will be searched for the translations.

    :param get_locale: a callable that returns the current locale

    :param get_timezone: a callable that returns the current timezone

    :param default_locale: default locale (as a string or as a
        Babel.Locale instance). This value will be accepted
        without checking if it's available.

    :param default_timezone: default timezone (as a string or as a
        `datetime.tzinfo` instance).

    :param markup: overwrite the function used by `translate` to flags HTML
        code as 'safe'. `markupsafe.Markup` is used by default.

    :param date_formats: update the defaults date formats.

    """

    def __init__(self, *args, **kwargs):
        super(Allspeak, self).__init__(*args, **kwargs)

    def __repr__(self):
        return '{}()'.format(self.__class__.__name__)



# coding=utf-8
from __future__ import print_function

from allspeak import integrations

from .conftest import (
    make_werkzeug_request, make_webob_request, make_django_request,
    get_test_request,
)


def test_get_werkzeug_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_werkzeug_request, headers=headers)
    langs = integrations.get_werkzeug_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_get_webob_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_webob_request, headers=headers)
    langs = integrations.get_webob_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']


def test_get_django_preferred_locales():
    headers = [('Accept-Language', 'fr; q=1.0, es; q=0.5, pt; q=0.6')]
    req = get_test_request(make_django_request, headers=headers)
    langs = integrations.get_django_preferred_locales(req)
    assert langs == ['fr', 'pt', 'es']



# coding=utf-8
import datetime
from babel import Locale, UnknownLocaleError
from babel.dates import get_timezone, UTC

from ._compat import string_types


__all__ = [
    'LOCALES_FOLDER', 'DEFAULT_LOCALE', 'DEFAULT_TIMEZONE',
    'normalize_locale', 'normalize_timezone',
    'split_locale', 'locale_to_str',
]

LOCALES_FOLDER = 'locales'

DEFAULT_LOCALE = 'en'
DEFAULT_TIMEZONE = UTC


def normalize_locale(locale):
    if not locale:
        return
    if isinstance(locale, Locale):
        return locale

    locale = split_locale(locale)

    if isinstance(locale, (tuple, list)):
        try:
            if len(locale) == 1:
                return Locale(locale[0].lower())
            else:
                return Locale(locale[0].lower(), locale[1].upper())
        except UnknownLocaleError:
            return None
    return None


def normalize_timezone(tzinfo):
    if not tzinfo:
        return
    if isinstance(tzinfo, datetime.tzinfo):
        return tzinfo
    try:
        return get_timezone(tzinfo)
    except LookupError:
        return


def split_locale(locale):
    """Returns a tuple (language, TERRITORY) or just (language, )
    from a a :class:`babel.core.Locale` instance or a string like `en-US` or
    `en_US`.
    """
    if isinstance(locale, Locale):
        tloc = [locale.language.lower()]
        if locale.territory:
            tloc.append(locale.territory.upper())
        return tuple(tloc)

    if isinstance(locale, string_types):
        locale = locale.replace('-', '_').lower().strip()
        tloc = locale.split('_')
        if len(tloc) > 1:
            tloc[-1] = tloc[-1].upper()
        return tuple(tloc)

    return locale


def locale_to_str(locale):
    return '_'.join(split_locale(locale))


def _flatten(dic):
    """Flatten a dictionary, separating keys by dots.

    >>>> dic = {
        'a': 1,
        'c': {
            'a': 2,
            'b': {
                'x': 5,
                'y' : 10,
            }
        },
        'd': [1, 2, 3],
    }
    >>>> flatten(dic)
    {'a': 1, 'c.a': 2, 'c.b.x': 5, 'c.b.y': 10, 'd': [1, 2, 3]}

    """
    def items():
        for key, value in dic.items():
            if isinstance(value, dict):
                for subkey, subvalue in _flatten(value).items():
                    yield str(key) + '.' + str(subkey), subvalue
            else:
                yield key, value

    return dict(items())


def _is_sequence(arg):
    return not hasattr(arg, "strip") and (
        hasattr(arg, "__getitem__") or hasattr(arg, "__iter__")
    )



# coding=utf-8
from __future__ import print_function

from os.path import join, dirname, abspath

from babel import Locale
from babel.dates import UTC
from markupsafe import Markup

from allspeak import I18n


LOCALES_TEST = abspath(join(dirname(__file__), u'locales'))
LOCALES_TEST2 = LOCALES_TEST + u'2'


def test_init_i18n():
    i18n = I18n(LOCALES_TEST)
    assert i18n.default_locale == Locale('en')
    assert i18n.default_timezone == UTC
    assert repr(i18n) == 'I18n()'


def test_init_i18n_default_locale():
    i18n = I18n(LOCALES_TEST, default_locale='es-PE')
    locale = Locale('es', 'PE')
    assert i18n.default_locale == locale
    assert i18n.get_locale() == locale


def test_init_i18n_locales():
    i18n = I18n(LOCALES_TEST)
    assert i18n.reader.folderpath == [LOCALES_TEST]

    folderpath = join(LOCALES_TEST, u'sub')
    i18n = I18n(folderpath)
    assert i18n.reader.folderpath == [folderpath]


def test_load_translations():
    i18n = I18n(LOCALES_TEST)
    i18n.load_translations()
    trans = i18n.translations
    print(trans)

    assert trans['en']['greeting'] == u'Hello World!'
    assert trans['es']['greeting'] == u'Hola mundo'
    assert trans['es_PE']['greeting'] == u'Habla'


def test_get_translations_from_locale():
    i18n = I18n(LOCALES_TEST)

    ltrans_es = i18n.get_translations_from_locale(Locale('es'))
    assert len(ltrans_es) == 1
    expected_es = 'foo cat greeting accented so'.split()
    trans_es = ltrans_es[0]
    assert sorted(trans_es.keys()) == sorted(expected_es)

    ltrans_espe = i18n.get_translations_from_locale(Locale('es', 'PE'))
    assert len(ltrans_espe) == 2
    expected_es = 'foo cat greeting accented so'.split()
    expected_espe = 'greeting'.split()
    trans_espe = ltrans_espe[0]
    trans_es = ltrans_espe[1]
    assert sorted(trans_espe.keys()) == sorted(expected_espe)
    assert sorted(trans_es.keys()) == sorted(expected_es)

    ltrans_en = i18n.get_translations_from_locale(Locale('en'))
    expected_en = 'foo cat greeting apple with_html'.split()
    trans_en = ltrans_en[0]
    assert sorted(trans_en.keys()) == sorted(expected_en)


def test_key_lookup():
    i18n = I18n(LOCALES_TEST)

    locale = Locale('es')
    assert i18n.key_lookup(locale, 'greeting') == u'Hola mundo'
    assert i18n.key_lookup(locale, 'so.much.such') == u'wow'

    locale = Locale('es', 'PE')
    assert i18n.key_lookup(locale, 'greeting') == u'Habla'

    locale = Locale('en')
    expected = {
        'zero': 'No apples',
        'one': 'One apple',
        'many': '{count} apples',
    }
    assert i18n.key_lookup(locale, 'apple') == expected

    # Key not found
    assert i18n.key_lookup(locale, 'this.is.wrong') is None

    # Language not found
    assert i18n.key_lookup(Locale('fr'), 'greeting') is None


def test_translate():
    i18n = I18n(LOCALES_TEST, default_locale='es-PE')

    assert i18n.translate('greeting') == u'Habla'

    locale = Locale('es')
    assert i18n.translate('greeting', locale=locale) == u'Hola mundo'

    assert i18n.translate('bla', locale=locale) == '<missing:bla/>'

    locale = Locale('en')
    assert i18n.translate('with_html', locale=locale) == Markup(u'<b>Hello</b>')


def test_translate_pluralize():
    i18n = I18n(LOCALES_TEST, default_locale='es-PE')
    locale = Locale('en')

    assert i18n.translate('apple', 0, locale=locale) == u'No apples'
    assert i18n.translate('apple', 1, locale=locale) == u'One apple'
    assert i18n.translate('apple', 10, locale=locale) == u'10 apples'


def test_multiple_sources():
    i18n = I18n([LOCALES_TEST, LOCALES_TEST2], default_locale='es')

    assert i18n.translate('greeting') == u'¡Bienvenidos!'
    assert i18n.translate('foo') == u'lorem ipsum'


def test_lazy_translate():
    i18n = I18n(LOCALES_TEST, default_locale='es_PE')

    lazy = i18n.lazy_translate('greeting')
    assert lazy != u'Habla'
    assert repr(lazy) == u'Habla'

    lazy = i18n.lazy_translate('bla')
    assert repr(lazy) == '<missing:bla/>'

    locale = Locale('en')
    lazy = i18n.lazy_translate('greeting', locale=locale)
    assert lazy != u'Hello World!'
    assert repr(lazy) == u'Hello World!'

    locale = Locale('fr')
    lazy = i18n.lazy_translate('greeting', locale=locale)
    assert lazy != '<missing:greeting/>'
    assert repr(lazy) == '<missing:greeting/>'


def test_for_incomplete_locales():
    i18n = I18n(LOCALES_TEST)
    assert i18n.test_for_incomplete_locales()

    i18n.translations = {
        'es': {
            'a': 1,
            'b': {0: 'nope', 1: 'one'},
            'c': 1,
        },
        'en': {
            'b': {0: 'nope'},
            'c': 1,
        },
        'fr': {
            'a': 1,
            'b': {0: 'nope'},
            'c': 1,
        },
        'pt': {
            'a': 1,
            'b': {0: 'nope', 1: 'one'},
            'c': 1,
        },
    }

    expected = {
        'en': set('a b.1'.split()),
        'fr': set('b.1'.split()),
    }
    assert i18n.test_for_incomplete_locales() == expected

    expected = {
        'en': set(['a']),
    }
    assert i18n.test_for_incomplete_locales('en', 'fr') == expected

    expected = {}
    assert i18n.test_for_incomplete_locales('es', 'pt') == expected


def test_content_negotiation(make_req):
    i18n = I18n(LOCALES_TEST)
    available_locales = ['en', 'es', 'es-PE']

    preferred = ['fr', 'es', 'pt']
    i18n._set_available_locales(available_locales)
    assert i18n.negotiate_locale(preferred) == Locale('es')

    preferred = ['fr', 'en-US', 'pt']
    i18n._set_available_locales(available_locales)
    assert i18n.negotiate_locale(preferred) == Locale('en')

    preferred = ['fr', 'es-PE', 'pt']
    i18n._set_available_locales(available_locales)
    assert i18n.negotiate_locale(preferred) == Locale('es', 'PE')

    preferred = ['fr', 'es-PE', 'pt']
    i18n._set_available_locales(['ru'])
    assert i18n.negotiate_locale(preferred) is None

    preferred = ['martian', 'venus', 'pt']
    i18n._set_available_locales(['klingon'])
    assert i18n.negotiate_locale(preferred) is None

    preferred = ['fr', 'es-PE', 'pt']
    i18n._set_available_locales([])
    assert i18n.negotiate_locale(preferred) is None
